<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 198 $
 * $Author: PragmaMx $
 * $Date: 2021-08-30 20:38:39 +0200 (Mo, 30. Aug 2021) $
 */

/**
 * pmxPlugin
 *
 * @package pragmaMx
 * @author Olaf
 * @copyright Copyright (c) 2014
 * @version $Id: Plugin.php 198 2021-08-30 18:38:39Z PragmaMx $
 * @access public
 */
 
 defined('mxMainFileLoaded') or die('access denied');
 
 // TODO: Admin.news. separat anbinden 
 
class pmxPlugin {
 
    private $_pluginname = 'undefined';
    
    private $_plugintype = array();
    Private $_plugins=false;
    
    private $_props = array();/* Array wird über setter dynamisch verändert */
    private $is_backend = false;
	private static $defaultRecord = array(
											/*  für Pluginliste */
						'name' => '',	        /* angezeigter Name */
						'plugin_name' => '',	/* Name des Ordners */
						'description' => '',	/* Beschreibung */
						'image' => "",			/* Bild */
						'type' => '',			/* type des Plugins - entspricht dem aufgerufenen plugin-Namen */
						'admin_menu' => 0,		/* Adminbereich vorhanden ? */
						'processing'=>0,	    /* 0=Frontend, 1=Backend,2=beide */
						'settings' =>array(),	/* Einstellungen des Plugins */
						'url' =>'',             /* Pfad zum Urheber*/
                        'version'=>'',          /* Version */
                        'position' => 0,        /* Reihenfolge der Abarbeitung */
                        'active' => 0,          /* active : Yes/No 1/0 */
						);	
                        
    private static $_plugintable = "plugins";         /* Tabellenname */
    
    
    /* Kennzeichnung der für die Plugins generierten Cache Dateien */
    const cache_extension = '.plugins.php';
    
    
    /**
     * pmxPlugin::__Construct()
     */
    function __construct($name="")
    {
        if (preg_match('#[^a-z0-9.-_]#', $name)) {
            trigger_error('\'' . $name . '\' is not a valid name for a plugin', E_USER_NOTICE);
            return false;
        }
        /* nachschauen, ob Admin.php geladen wurde, dann sind wir im Backend) */
        $this->is_backend = (defined('mxAdminFileLoaded'))?true:false; 
        
        /* plugin-namen zwischenspeichern */
        $this->_pluginname = $name;
    }
	

    /**
     *  @function run	
     *  
     *  @user  oherf
     *  @date  02/02/20
     *  @param [in] $outcome $
     *  @return $
     *  
     *  
     */
    public function run(&$outcome = null)
    {
		
		//if (pmxBase::get('plugins') ==0) return $outcome;
        /* active Plugins aus db/Cache einlesen */
        
        $files = $this->_get_plugins_from_db($this->_pluginname);
        //mxDebugFuncVars($files); 
        if ($files) {
               
            /* eventuelle ausgaben abfangen */
            ob_start();
            
            /* hier wird gleich der Modulname an die plugin-Datei übergeben !! */
			
            foreach ($files as $key) {
                $pmx_plugin = null;
                //mxDebugFuncVars($this->is_backend,$key);
                switch (true) {
                    
                   case $this->is_backend==1 and $key['processing']==0 :  /* nur im Frontend ausführen */
                    break;
                   case $this->is_backend==false and $key['processing']==1 :  /* nur im Backend ausführen */
                    break;
                    
                   default:
                    
                    $filename=PMX_PLUGIN_DIR . DS .  $key['plugin_name'] . DS . 'core' . DS . $key['type'] . ".php";
                    $plugin_name = $key['plugin_name'];
                    $this->__set('settings',$key['settings']);

                    switch (true) {
                        case !file_exists($filename):
                            trigger_error('not found plugin-file: ' . mx_strip_sysdirs($filename), E_USER_NOTICE);
                            break;                        
			case !include( $filename ):
                            trigger_error('not included plugin-file: ' . mx_strip_sysdirs($filename), E_USER_NOTICE);
                            break;
                        case !$pmx_plugin:
                            trigger_error('pmx_plugin not found in plugin-file: ' . mx_strip_sysdirs($filename), E_USER_NOTICE);
                            break;
                        case !is_callable($pmx_plugin): 
                            //mxDebugFuncVars($filename);
                            trigger_error('pmx_plugin not callable in plugin-file: ' . mx_strip_sysdirs($filename), E_USER_NOTICE);
                            break;
                        default:
                            /* anonyme Funktion $pmx_plugin in der Plugin-Datei ausführen */
                            //mxDebugFuncVars($key);
                            $pmx_plugin($plugin_name, $this->_props, $outcome);
                    }
                }
            }
            /* Ausgabepuffer wieder löschen */
            ob_get_clean();
            
        } else {
            return false;
        }        
        
		return $outcome;
	}

    /**
     * Plugin.php:_plugins
     * alle vorhandenen Plugins ermitteln
     *
     * @return array
     */
    function get_plugins()
    {
        if (!$this->_checkAccess()) return false;
        $new_plugins = array() ; /* neue Plugins */
        
        /* plugins aus den Ordnern lesen */        
        $f_plugins = $this->_get_plugins_from_files();   

        /* alle Daten aus der DB holen */
        $db_plugins = $this->get_all_plugins_from_db();
        
        /* vergleichen*/
        $fplugins=array();
        foreach ($f_plugins as $f_key => $f_type) {
            /* erst dir Plugin-Typen der Files durchlaufen */  
            //mxDebugFuncVars($f_type);
            if (!array_key_exists($f_key,$db_plugins)) {
                
                /* wenn noch kein Plugin unter diesem Type angemeldet ist, dann eintragen */
                foreach ($f_type as $plugin) {
                    //
                    $this->_addPlugin($plugin);
                }
            } else {
                $ftype=$f_key;
                foreach ($f_type as $p_key=>$plugin) {
                    /* in $plugin stehen jetzt die Daten der einzelnen Plugins */
                    
                    $f_plugins[$ftype][$p_key]['new']=true; /* false, wenn das Plugin bereits in der DB vorhanden */
                    
                    foreach($db_plugins[$plugin['type']] as $d_key=>$dplugin) {
                        /* jetzt vergleichen zwischen den DB-Plugins und den File-Plugins */
                     
                        if ($plugin['plugin_name']==$dplugin['plugin_name']) {
                            /* wenn File-Plugin in der DB, dann markieren */
                            $f_plugins[$f_key][$p_key]['new']=false;
                        
                        }
						//mxDebugFuncVars($d_key,$dplugin);
                        /* jetzt noch überprüfen, ob das Plugin, welches in der DB steht noch gültig ist */
                        
						/* wenn Falsch , dann ist das Plugin n.i.O., also löschen */
						
						$filename=PMX_PLUGIN_PATH .  $dplugin['plugin_name'] . DS . 'core' . DS . $dplugin['type'] . ".php";
						//mxDebugFuncVars($filename);
						switch (true) {
							case !file_exists($filename):             
							case !include( $filename ):
							case !is_callable($pmx_plugin): 
								
								$f_plugins[$ftype][$p_key]['new']==false; /* und markieren */
								$this->_delPlugin($dplugin);
								break;
							default:

						} 							
                    
                    }
					
                    /* Wenn Flag immer noch TRUE, dann Plugin eintragen */
                    if ($f_plugins[$ftype][$p_key]['new']==true ){
                        $this->_addPlugin($f_plugins[$ftype][$p_key]);
                    }
                    
                }
            }				


            /* jetzt noch Datenbank renummerieren */
            $this->_renumPosition($f_key);
        }
		
		/* prüfen, ob alle Plugins, welche in der DB stehen, auch noch vorhanden sind */
		foreach ($db_plugins as $key=>$value){
			
			$query = "select id,plugin_name,type from ". 
			pmxBase::prefix() . "_" . self::$_plugintable . " ";

			$filesdb = pmxDatabase::system_query($query);
			while ( $plugin= pmxDatabase::fetch_assoc($filesdb)) {
				$filename= PMX_PLUGIN_PATH .  $plugin['plugin_name'] . DS . 'core' . DS . $plugin['type'] . ".php";
				/* wenn nicht mehr vorhanden, aus der DB löschen */
				if (!file_exists($filename)){
					$this->_delPlugin($plugin);
					$this->_renumPosition($plugin);
				}
			}
		}        
		
		/* nun unbedingt kompletten Cache löschen */
		$cache = load_class('Cache');
		$cache->truncate();
		unset ($cache);
		
        /* jetzt alle aktualisierten Daten wieder aus der DB holen */
        $db_plugins = $this->get_all_plugins_from_db();    
        
        return $db_plugins;
    }
    /**
     *  get_types $
     *  
     *  @return $
     *  
     *  Date: 2020-02-19T19:03:44 $
     *  created by : Olaf Herfurth $
     */
    function get_types()
    {
        $db_types=array();
           
        $query = "select type from ". 
                pmxBase::prefix() . "_" . self::$_plugintable . " GROUP BY type";  
        $filesdb = pmxDatabase::system_query($query);      
        while ( $types= pmxDatabase::fetch_row($filesdb)) {
            $db_types[]=$types; 
        }  
        return $db_types;
    }
    
    /**
     *  get_all_plugins_from_db $
     *  
     *  @param [in] $plugintype $
     *  @return $
     *  
     *  Date: 2020-02-19T19:03:53 $
     *  created by : Olaf Herfurth $
     */
    function get_all_plugins_from_db($plugintype=false)
    {        
        static $me_plugins;

        if (is_array($me_plugins)) return $me_plugins;
        
        $fWhile=(!$plugintype)?"":" where type='".$plugintype."'";
           /* Daten aus DB holen */
        $query = "select * from ". 
                pmxBase::prefix() . "_" . self::$_plugintable . $fWhile . " order by type, position asc";
        
        $filesdb = pmxDatabase::system_query($query);
        $db_plugins = array();
        
            while ( $plugin= pmxDatabase::fetch_assoc($filesdb)) {
				//
                $plugin['settings']=unserialize($plugin['settings']);
                $me_plugins[$plugin['type']][$plugin['id']]=$plugin;
				if ($plugin['settings']===false) $plugin['settings']=array();//mxDebugFuncVars($plugin);
                
            } 
        
        return $me_plugins;
    }
    
    /**
     *  _get_plugins_from_db $
     *  
     *  @param [in] $pluginname $
     *  @return $
     *  
     *  Date: 2020-02-19T19:04:01 $
     *  created by : Olaf Herfurth $
     */
    private function _get_plugins_from_db($pluginname=false) {
        
        $cache = load_class('Cache');
        $cache->sFileExtension = self::cache_extension;
        
        if (($files = $cache->read("plugin")) === false) { 
            /* query für db-Abfrage vorbereiten */
			pmxBase::set('mxPluginActive',true); /* plugins abschalten */
            $query = "select * from ". 
                    pmxBase::prefix() . "_" . self::$_plugintable . "
                    where active=1 order by type, position asc";
            
            $filesdb = pmxDatabase::system_query($query);;
             
            while ( $plugin= pmxDatabase::fetch_assoc($filesdb)) {
                $plugin['settings']=unserialize($plugin['settings']);
                $files[$plugin['type']][$plugin['id']]=$plugin;
                
            }

            //mxDebugFuncVars($files);
            $cache->write($files, "Plugin", 18000); // 5 Stunden Cachezeit
        }
		pmxBase::set('mxPluginActive',false); /* plugins einschalten */
        //mxDebugFuncVars($files);
        if (!isset($files[$pluginname])) {
            return false;
        }
        if ($pluginname==false) {
            return $files;
        }		
        
        return $files[$pluginname];
    }

	/**
	 *   Plugin.php:_get_plugins
	 *  
	 *  @return 
	 *  
	 */
	protected function _get_plugins_from_files()
	{
		/* alle Plugin-Dateien einlesen */

        //$plugins = self::_plugins();
            
        /* alle Plugins aus den Pfad lesen */
            
        $f_plugins = array();
        foreach ((array)glob(PMX_PLUGIN_PATH .  '*' . DS . 'setup.pmx.php', GLOB_NOSORT) as $filename) {
            if ($filename) {
                $plug_index = dirname($filename).DS.'index.php';
                //mxDebugFuncVars($plug_index);
                if (file_exists($plug_index)) {
                    $plugin=array();
                    $pmx_plugin=false;
                    switch (true) {

                        case !include( $plug_index ):
                        case !$plug_index:
                        case !is_callable($pmx_plugin):
                            /* TODO: undefinierte Plugins aus der DB entfernen */
                            trigger_error('Invalid plugin-file: ' . mx_strip_sysdirs($plug_index), E_USER_NOTICE);
                            break;
                        default:
                            /* anonyme Funktion $pmx_plugin in der Plugin-Datei ausführen */
                            $pmx_plugin(basename(dirname($plug_index)), $plugin);

                            foreach ($plugin as $pval) {
                                $f_plugins[$pval['type']][]=$pval;
                            }
                    }                            
                    
                }
            }
        }
        //mxDebugFuncVars($f_plugins);       
        return $f_plugins;
	}
	
	/**
	 *   Plugin.php:_addPlugin
	 *  
	 *  @param [in] array Description
	 *  @return 
	 *  
	 */
	private function _addPlugin ($plugin=false)
	{
		if ((!$plugin) or (!is_array($plugin))) return false;
        
		$prefix=pmxBase::prefix();
		$plugin= self::_checkrecord($plugin);
		/* nachfolgende Angaben müssen immer vorhanden sein !! */
        if (trim($plugin['plugin_name'])=="") return false;
		if (trim($plugin['type'])=="") return false;
        
        /* prüfen, ob Angaben stimmen */
        //
        if ($this->_check_plugin_file($plugin))
        {
			//mxDebugFuncVars($plugin);
            /* TODO aktuelle Position ermitteln und hinten dran setzen*/
            
            $data= pmxDatabase::system_query("Select id from ".pmxBase::prefix() . "_" . self::$_plugintable." where type='".$plugin['type']."'");
            $count=pmxDatabase::num_rows($data);
            
            
            /* Datensatz zusammensetzen und Speichern */
            
                $qryarr=array(
                    "name"=>mxAddSlashesForSQL($plugin['name']),
                    "plugin_name"=>$plugin['plugin_name'],
                    "description"=>mxAddSlashesForSQL($plugin['description']),
                    "type"=>mxAddSlashesForSQL($plugin['type']),
                    "image"=>mxAddSlashesForSQL($plugin['image']),
                    "url"=>mxAddSlashesForSQL($plugin['url']),
                    "settings"=>mxAddSlashesForSQL(serialize($plugin['settings'])),
                    "processing"=>intval($plugin['processing']),
                    "version"=>mxAddSlashesForSQL($plugin['version']),
                    "active"=>0,
                    "position"=>($count+1),
					"admin_menu"=>$plugin['admin_menu'],
                    );
                    $qry ="INSERT IGNORE INTO ". pmxBase::prefix() . "_" . self::$_plugintable . " SET ";
                    $qryset=array();
                    foreach ($qryarr as $key => $value) {
                        $qryset[]=$key."='".$value."'";
                    }
                    $qry .=implode(",",$qryset);
                    //mxDebugFuncVars($qry);
                    pmxDatabase::system_query($qry);
					
		}	
		
		return true;
	}
    
    /**
     *  @_delPlugin	
     *  
     *  @user  oherf
     *  @date  02/05/20
     *  @return $
     *  
     *  
     */
    private function _delPlugin ($plugin=array())
    {
        if (!array_key_exists('id',$plugin)) return false;
        $query = "delete from ". pmxBase::prefix() . "_" . self::$_plugintable . " where id=".$plugin['id'];
        pmxDatabase::system_query($query);   
        
        // Renummerieren der Positionen
        $this->_renumPosition($plugin['type']);
        return true;
    }
    /**
     *  get_PluginFromId $
     *  
     *  @param [in] $plugin $
     *  @return $
     *  
     *  Date: 2020-02-19T19:04:29 $
     *  created by : Olaf Herfurth $
     */
    function get_PluginFromId($plugin)
    {
        if (!$this->_checkAccess()) return false;
        $key=intval($plugin);
        $query = "select * from ". pmxBase::prefix() . "_" . self::$_plugintable . " where id=".$key. " ";
        $result=pmxDatabase::system_query($query); 
        
        return pmxDatabase::fetch_assoc($result);        
    }	
	
    /**
     *  getPlugInfo 
     *  
     *  @param [in] $plugin 
     *  @return 
     *  
     *  Date: 04/21/21
     *  created by : oherf 
     */
    function getPluginInfo($plugin)
    {
        if (!$this->_checkAccess()) return false;
        
        $result=$this->get_PluginFromName($plugin); 
        
        return $result;        
    }	
    /**
     *  get_PluginFromName $
     *  
     *  @param [in] $plugin $
     *  @param [in] $type   $
     *  @return $
     *  
     *  Date: 2020-02-19T19:04:33 $
     *  created by : Olaf Herfurth $
     */
    function get_PluginFromName($plugin,$type=false)
    {
        if (!$this->_checkAccess()) return false;
        
        $query = "select * from ". pmxBase::prefix() . "_" . self::$_plugintable . " where plugin_name='".$plugin."'";
		$query .=($type)?" and type='".$type."'":"";
        $result=pmxDatabase::system_query($query); 
        
        return pmxDatabase::fetch_assoc($result);        
    }	
	
    /**
     *  @setPluginActive	
     *  
     *  @user  oherf
     *  @date  02/05/20
     *  @param [in] $plugin $
     *  @return $
     *  
     *  
     */
    function setPluginActive( $plugin=array())
    {
        if (!$this->_checkAccess()) return false;
        if (!array_key_exists('id',$plugin)) return false;
        $query = "update  ". pmxBase::prefix() . "_" . self::$_plugintable . "  set active='1' where id=".$plugin['id']. "";
        pmxDatabase::system_query($query); 
        /* Datei activate.php im Plugin ausführen */
        $filename=PMX_PLUGIN_PATH .  $plugin['plugin_name'] . DS . 'core' . DS . "activate.php";
        switch (true) {
            case !file_exists($filename):
            case !include( $filename ):
                return false;
                break;
        }
        return true;        
    }
    
    /**
     *  @setPluginInActive	
     *  
     *  @user  oherf
     *  @date  02/05/20
     *  @return $
     *  
     *  
     */
    function setPluginInActive($plugin=array())
    {
        if (!$this->_checkAccess()) return false;
        if (!array_key_exists('id',$plugin)) return false;
        $query = "update  ". pmxBase::prefix() . "_" . self::$_plugintable . "  set active='0' where id=".$plugin['id']. "";
        pmxDatabase::system_query($query);   
        return true;        
    }
    
    /**
     *  @updatePlugin	
     *  
     *  @user  oherf
     *  @date  02/05/20
     *  @return $
     *  
     *  
     */
    function updatePlugin($plugin=array())
    {
        if (!$this->_checkAccess()) return false;
		if ((!$plugin) or (!is_array($plugin))) return false;
        if (!array_key_exists('id',$plugin)) return false;
        $id=$plugin['id'];
		$prefix=pmxBase::prefix();
		$plugin= self::_checkrecord($plugin);
		/* nachfolgende Angaben müssen immer vorhanden sein !! */
        if (trim($plugin['plugin_name'])=="") return false;
		if (trim($plugin['type'])=="") return false;
        
        /* prüfen, ob Angaben stimmen */
        
        if ($this->_check_plugin_file($plugin)){
			//mxDebugFuncVars($plugin);
            $qryarr=array(
                "name"=>$plugin['name'],
                "plugin_name"=>$plugin['plugin_name'],
                "description"=>mxAddSlashesForSQL($plugin['description']),
                "type"=>mxAddSlashesForSQL($plugin['type']),
                "image"=>mxAddSlashesForSQL($plugin['image']),
                "url"=>mxAddSlashesForSQL($plugin['url']),
                "processing"=>intval($plugin['processing']),
                "settings"=>mxAddSlashesForSQL(serialize($plugin['settings'])),
                "position"=>intval($plugin['position']),
                );
                $qry ="UPDATE ". pmxBase::prefix() . "_" . self::$_plugintable . " SET ";
                $qryset=array();
                foreach ($qryarr as $key => $value) {
                    $qryset[]=$key."='".$value."'";
                }
                $qry .=implode(",",$qryset);
                $qry .=" WHERE id='".$id."'";
                //mxDebugFuncVars($qry);
                if ($qry) pmxDatabase::system_query($qry);
                $this->_renumPosition($plugin['type']);
		}	
        return true;        
    }
    
    /**
     *  updatePluginPosition $
     *  
     *  @param [in] $pluginid $
     *  @param [in] $position $
     *  @return $
     *  
     *  Date: 2020-02-19T19:04:45 $
     *  created by : Olaf Herfurth $
     */
    function updatePluginPosition($pluginid=false,$position=NULL)
    {
        if (!$this->_checkAccess()) return false;
		if (!$pluginid) return false;
        if (INTVAL($pluginid)==0) return false;
        
		$prefix=pmxBase::prefix();
		
        $plugin = $this->get_PluginFromId($pluginid);
        
        /* prüfen, ob Angaben stimmen */
        
        if ($this->_check_plugin_file($plugin)){

                $qry ="UPDATE ". pmxBase::prefix() . "_" . self::$_plugintable . " SET position='".$position."'";
                $qry .=" WHERE id=".$plugin['id']."";
                //mxDebugFuncVars($qry);
                if ($qry) pmxDatabase::system_query($qry);
                $this->_renumPosition($plugin['type']);
		}	
        return true;        
    }
    /**
     *  @_renumPosition	
     *  
     *  @user  oherf
     *  @date  02/06/20
     *  @param [in] $type $
     *  @return $
     *  
     *  
     */
    private function _renumPosition ($type=false)
    {
        /* renummerieren der Positionsangaben in der Datenbank */
        switch (true) {
            case !$type:
            case is_array($type):
            case !is_string($type):
            return false;
            break;
        }
        
        $qry= "SELECT * from ". pmxBase::prefix() . "_" . self::$_plugintable . " WHERE type='". $type ."' ORDER BY position";
        $result=pmxDatabase::system_query($qry);
        $counter=0;
        while ($data = pmxDatabase::fetch_assoc($result))
        {
            $counter++;
            pmxDatabase::system_query( "UPDATE ". pmxBase::prefix() . "_" . self::$_plugintable . "  SET position='".$counter."' WHERE id='".$data['id']."'");
        }
        return $counter;
    }
    
    
	/**
	 *   Plugin.php:_checkRecord
	 *  
	 *  @param [in] $record Description
	 *  @return 
	 *  
	 */
	private function _checkRecord($record) {
		
        $record= array_intersect_key($record,self::$defaultRecord);
		return array_merge (self::$defaultRecord,$record);
	}
    
    /**
     *  _check_plugin_file $
     *  
     *  @return $
     *  
     *  Date: 2020-02-19T19:05:07 $
     *  created by : Olaf Herfurth $
     */
    private function _check_plugin_file($plugin=array()) {
        
        $plugin= $this->_checkRecord($plugin);
        
        $filename=PMX_PLUGIN_PATH .  $plugin['plugin_name'] . DS . 'core' . DS . $plugin['type'] . ".php";
		//mxDebugFuncVars($filename,file_exists($filename),(include( $filename )),is_callable($pmx_plugin));
        switch (true) {
            case !file_exists($filename):             
            case !include( $filename ):
            case !is_callable($pmx_plugin): 
                //mxDebugFuncVars($filename);
                return false;
                break;
            default:
                //mxDebugFuncVars($filename);
               return true;
			   
        }        
        return false;
    }
   /**
     * ::__get()
     *
     * @param mixed $name
     * @return
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->_props)) {
            return $this->_props[$name];
        }
        $trace = debug_backtrace();
        trigger_error('undefined property \'' . $name . '\' in ' . mx_strip_sysdirs($trace[0]['file']) . ' line ' . $trace[0]['line'], E_USER_NOTICE);
        return null;
    }

    /**
     * ::__set()
     *
     * @param string $name
     * @param mixed $value
     * @return
     */
    public function __set($name, $value)
    {
        $this->_props[$name] = $value;
    }

    /**
     * ::set()
     *
     * @param mixed $name
     * @param mixed $value
     * @return
     */
    public function set($name, $value = null)
    {
        if (is_array($name)) {
            foreach ($name as $key => $value) {
                $this->_props[$key] = $value;
            }
        } else {
            $this->_props[$name] = $value;
        }
    }
    /**
     *  _checkAccess $
     *  
     *  @return $
     *  
     *  Date: 2020-02-19T19:05:18 $
     *  created by : Olaf Herfurth $
     */
    private function _checkAccess()
    {
        /* nur Backend und als Admin */
		//mxDebugFuncVars($this->is_backend,MX_IS_ADMIN);
        if ($this->is_backend && MX_IS_ADMIN) return true;
        return false;
    }
	
	private function _query ($query) 
	{
		pmxBase::set('mxPluginActive',true);
		
	}
}

/**
 *   mxGetModuleInfo
 *  TODO: noch nicht für Admin-Module möglich
 *  @param $module_name - Modulename
 *  @return
 *
 */
 function mxGetPluginInfo ($module_name)
{
	$tmp = new pmxPlugin();
	return $tmp->getPluginInfo($module_name);
}
?>